@@ -64,10 +64,20 @@
 #' @param seed an integer value containing the random number generator state. This
 #' argument can be used to replicate the results of a GA search. Note that
 #' if parallel computing is required, the doRNG package must be installed.
+#' (Default: 'seed = NULL')
+#' @param monitor a logical or an R function which takes as input the current
+#'                state of the 'ga-class' object and show the evolution of the
+#'                search. By default, 'monitor = FALSE' so any
+#'                output is suppressed. Possible also, the functions
+#'                'gaMonitor' or 'gaMonitor2' (depending on whether or not is
+#'                an RStudio session) which print the average and best fitness
+#'                values at each iteration. If set to 'plot' these information
+#'                are plotted on a graphical device. Other functions can be
+#'                written by the user and supplied as argument.
 #' @param plot logical; Plot restructured counts with fitted lines using
 #' \code{\link{plot.lfq}} and \code{\link{lfqFitCurves}} (default : FALSE).
 #' @param plot.score logical; Plot genetic algorithm fitness progression.
-#'    (Default: plot.score=TRUE)
+#'    (Default: plot.score=TRUE).
 #' @param ... additional parameters to pass to \code{\link[GA]{ga}}
 #'
 #'
@@ -128,7 +138,7 @@
 #' # compare fitness score (fESP) to
 #' # that calculated with "true" growth parameter values
 #' plot(output, draw = FALSE)
-#' lfqFitCurves(output, par=list(Linf=80, K=0.5, t_anchor=0.25, C=0.75, ts=0),
+#' lfqFitCurves(output, par=list(Linf=80, K=0.5, t_anchor=0.25, C=0.75, ts=0.5),
 #'        draw = TRUE, col=1, flagging.out = FALSE)$fESP
 #' lfqFitCurves(output, par=output$par, draw = TRUE, col=2, flagging.out = FALSE)$fESP
 #' legend("top", legend=c("orig.", "GA"), lty=2, col=1:2, ncol=2)
@@ -168,7 +178,8 @@ ELEFAN_GA <- function(
   addl.sqrt = FALSE,
   agemax = NULL,
   flagging.out = TRUE,
-  seed = 1,
+  seed = NULL,
+  monitor = FALSE,
   plot = FALSE,
   plot.score = TRUE,
   ...
@@ -246,11 +257,7 @@ ELEFAN_GA <- function(
   if(seasonalised){
     min = c(low_Linf, low_K, low_tanc, low_C, low_ts)
     max = c(up_Linf, up_K, up_tanc, up_C, up_ts)
-
-    writeLines(paste(
-      "Genetic algorithm is running. This might take some time.\n
-      A beep tone will alert completion."
-    ,sep=" "))
+    writeLines("Genetic algorithm is running. This might take some time.")
     flush.console()
     fit <- GA::ga(
       type = "real-valued",
@@ -261,6 +268,7 @@ ELEFAN_GA <- function(
       flagging.out = flagging.out,
       popSize = popSize, maxiter = maxiter, run = run, parallel = parallel,
       pmutation = pmutation, pcrossover = pcrossover, elitism = elitism,
+      seed = seed, monitor = FALSE,
       ...
     )
     pars <- as.list(fit@solution[1,])
@@ -268,9 +276,7 @@ ELEFAN_GA <- function(
   }else{
     min = c(low_Linf, low_K, low_tanc)
     max = c(up_Linf, up_K, up_tanc)
-
-    writeLines(paste(
-      "Genetic algorithm is running. \nThis will take some time. \nA beep tone will alert completion.",sep=" "))
+    writeLines("Genetic algorithm is running. This might take some time.")
     flush.console()
     fit <- GA::ga(
       type = "real-valued",
@@ -283,6 +289,7 @@ ELEFAN_GA <- function(
       popSize = popSize, maxiter = maxiter, run = run, parallel = parallel,
       pmutation = pmutation, pcrossover = pcrossover, elitism = elitism,
       seed = seed,
+      monitor = FALSE,
       ...
     )
     pars <- as.list(fit@solution[1,])
@@ -294,9 +301,6 @@ ELEFAN_GA <- function(
     GA::plot(fit)
   }
 
-  # notify completion
-  beepr::beep(10); beepr::beep(1)
-
   final_res <- lfqFitCurves(lfq = lfq, par=pars,
                             flagging.out = flagging.out,
                             agemax = agemax)
