@@ -46,6 +46,11 @@
 #'   \item \strong{ts} summer point (ts = WP - 0.5) (range: 0 to 1, default: 0);
 #' }
 #' @param SA_time numeric; Maximum running time in seconds (default : 60 * 1).
+#' @param maxit Integer. Maximum number of iterations of the
+#'              algorithm. Default is NULL.
+#' @param nb.stop.improvement Integer. The program will stop when
+#'              there is no any improvement in 'nb.stop.improvement'
+#'              steps. Default is NULL
 #' @param SA_temp numeric; Initial value for temperature (default : 1e5).
 #' @param verbose logical; TRUE means that messages from the algorithm
 #'    are shown (default : TRUE).
@@ -85,7 +90,7 @@
 #'
 #' # compare to original parameters
 #' tmp <- lfqFitCurves(output, col=4, lty=1,
-#'    par=list(Linf=80, K=0.5, t_anchor=0.25, C=0.75, ts=0), draw=TRUE)
+#'    par=list(Linf=80, K=0.5, t_anchor=0.25, C=0.75, ts=0.5), draw=TRUE)
 #' tmp$fESP
 #' output$Rn_max
 #' }
@@ -140,19 +145,23 @@
 #' annealing for global optimization: the GenSA Package. R Journal, 5(1), 13-28.
 #' @export
 
-ELEFAN_SA <- function(x,
-                      seasonalised = FALSE,
-                      init_par = list(Linf = 50, K = 0.5, t_anchor = 0.5, C = 0, ts = 0),
-                      low_par = NULL, #list(Linf = 1, K = 0.01, t_anchor = 0, C = 0, ts = 0),
-                      up_par = NULL, #list(Linf = 1000, K = 10, t_anchor = 1, C = 1, ts = 1),
-                      SA_time = 60 * 1,
-                      SA_temp = 1e5,
-                      verbose = TRUE,
-                      MA = 5, addl.sqrt = FALSE,
-                      agemax = NULL,
-                      flagging.out = TRUE,
-                      plot = FALSE,
-                      plot.score = TRUE){
+ELEFAN_SA <- function(
+  x,
+  seasonalised = FALSE,
+  init_par = list(Linf = 50, K = 0.5, t_anchor = 0.5, C = 0, ts = 0),
+  low_par = NULL,
+  up_par = NULL,
+  SA_time = 60 * 1,
+  maxit = NULL,
+  nb.stop.improvement = NULL,
+  SA_temp = 1e5,
+  verbose = TRUE,
+  MA = 5, addl.sqrt = FALSE,
+  agemax = NULL,
+  flagging.out = TRUE,
+  plot = FALSE,
+  plot.score = TRUE
+){
 
   res <- x
   classes <- res$midLengths
@@ -247,11 +256,23 @@ ELEFAN_SA <- function(x,
     return(-Lt$fESP)
   }
 
+
+    ## control list
+    control <- list(temperature = SA_temp,
+                    verbose = verbose)
+    if(!is.null(SA_time)) control$max.time = SA_time
+    if(!is.null(maxit)) control$maxit = maxit
+    if(!is.null(nb.stop.improvement)) control$nb.stop.improvement
+
+
   if(seasonalised){
     # Simulated annealing with seasonalised VBGF
     writeLines(paste(
-      "Simulated annealing is running. \nThis will take approximately", round(SA_time/60,digits=2),"minutes. \nA beep tone will alert completion."
-    ,sep=" "))
+      "Simulated annealing is running. \nThis will take approximately",
+      round(SA_time/60,digits=2),
+      "minutes.",
+      sep=" "
+    ))
     flush.console()
     SAfit <- GenSA::GenSA(
       par = c(init_Linf, init_K, init_tanc, init_C, init_ts),
@@ -260,11 +281,7 @@ ELEFAN_SA <- function(x,
       upper = c(up_Linf, up_K, up_tanc, up_C, up_ts),
       agemax = agemax,
       flagging.out = flagging.out,
-      control = list(
-        max.time = SA_time,
-        temperature = SA_temp,
-        verbose = verbose
-      ),
+      control = control,
       lfq = res
     )
 
@@ -272,7 +289,12 @@ ELEFAN_SA <- function(x,
     names(pars) <- c("Linf","K","t_anchor", "C", "ts")
   }else{
     # Simulated annealing
-    writeLines(paste("Simulated annealing is running. This takes", round(SA_time/60,digits=2),"minutes. A beep tone will inform you\n when the calculations are done.",sep=" "))
+    writeLines(paste(
+      "Simulated annealing is running. \nThis will take approximately",
+      round(SA_time/60,digits=2),
+      "minutes.",
+      sep=" "
+    ))
     flush.console()
     SAfit <- GenSA::GenSA(par = c(init_Linf, init_K, init_tanc),
       fn = SAfun,
@@ -280,11 +302,7 @@ ELEFAN_SA <- function(x,
       upper = c(up_Linf, up_K, up_tanc),
       agemax = agemax,
       flagging.out = flagging.out,
-      control = list(
-        max.time = SA_time,
-        temperature = SA_temp,
-        verbose = verbose
-      ),
+      control = control,
       lfq = res
     )
 
@@ -322,10 +340,6 @@ ELEFAN_SA <- function(x,
     par(op)
   }
 
-  # notify completion
-  beepr::beep(10); beepr::beep(1) # beepr::beep(2)
-
-
   final_res <- lfqFitCurves(lfq = res,par=pars,flagging.out = flagging.out,
                             agemax = agemax)
 
